@@ -12,40 +12,16 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-#  author: terry.yinzhe@gmail.com
+#  author: terry@odd-e.com
 #
 """
 lizard is a simple code complexity analyzer without caring about the C/C++
-header files or Java imports. It can deal with
-
-* Java
-* C/C++
-* Objective C.
-
-It counts
-
-* the nloc (net lines of code, excluding comments and blanks),
-* CCN (cyclomatic complexity number) or Modified CCN,
-* token count of functions.
-* parameter count of functions.
-
-You can set limitation for CCN (-C), the number of parameters (-a). Functions
-that exceed these limitations will generate warnings. The exit code of lizard
-will be none-Zero if there are warnings.
-
-This tool actually calculates how complex the code 'looks' rather than how
-complex the code real 'is'. People will need this tool because it's often very
-hard to get all the included folders and files right when they are complicated.
-But we don't really need that kind of accuracy when come to cyclomatic
-complexity.
-
-It requires python2.6 or above (early versions are not verified).
+header files or Java imports.
+Please find the README.md for more information.
 """
+VERSION = "1.8.0"
 
-VERSION = "1.7.9"
-
-
-import itertools, traceback
+import itertools
 import re
 
 
@@ -53,92 +29,92 @@ DEFAULT_CCN_THRESHOLD = 15
 
 
 def analyze(paths, options):
-    ''' This is the most important function of lizard.
-        It analyzes the given paths with the options.
-        Can be used directly by other Python application.
+    ''' returns an iterator of file infomation.
     '''
-
-    languages = LanguageChooser(
-            includeHashIfConditions = not options.no_preprocessor_count,
-            switchCasesAsOneCondition = options.switchCasesAsOneCondition,)
-
     files = FilesFilter(options.exclude, options.duplicates).getFileNames(paths)
-    fileAnalyzer = FileAnalyzer(languages, options.extensions)
+    fileAnalyzer = FileAnalyzer(options.extensions)
     return mapFilesToAnalyzer(files, fileAnalyzer, options.working_threads)
 
-def createCommandLineParser():
-    from optparse import OptionParser
-    parser = OptionParser(version=VERSION)
-    parser.add_option("-v", "--verbose",
+def createCommandLineParser(prog=None):
+    from argparse import ArgumentParser
+    parser = ArgumentParser(prog=prog)
+    parser.add_argument('paths', nargs='*', default=['.'], help='list of the filename/paths.')
+    parser.add_argument('--version', action='version', version=VERSION)
+    parser.add_argument("-V", "--verbose",
             help="Output in verbose mode (long function name)",
             action="store_true",
             dest="verbose",
             default=False)
-    parser.add_option("-C", "--CCN",
+    parser.add_argument("-C", "--CCN",
             help =  "Threshold for cyclomatic complexity number warning. "+
                     "The default value is %d. Functions with CCN bigger than this number will generate warning" % DEFAULT_CCN_THRESHOLD,
             action="store",
-            type="int",
+            type=int,
             dest="CCN",
             default=DEFAULT_CCN_THRESHOLD)
-    parser.add_option("-a", "--arguments",
+    parser.add_argument("-a", "--arguments",
             help="Limit for number of parameters",
             action="store",
-            type="int",
+            type=int,
             dest="arguments",
             default=100)
-    parser.add_option("-w", "--warnings_only",
+    parser.add_argument("-w", "--warnings_only",
             help="Show warnings only, using clang/gcc's warning format for printing warnings. http://clang.llvm.org/docs/UsersManual.html#cmdoption-fdiagnostics-format",
             action="store_true",
             dest="warnings_only",
             default=False)
-    parser.add_option("-i", "--ignore_warnings",
+    parser.add_argument("-i", "--ignore_warnings",
             help="If the number of warnings is equal or less than the number, the tool will exit normally, otherwize it will generate error. Useful in makefile when improving legacy code.",
             action="store",
-            type="int",
+            type=int,
             dest="number",
             default=0)
-    parser.add_option("-x", "--exclude",
+    parser.add_argument("-x", "--exclude",
             help="Exclude files that match this pattern. * matches everything, ? matches any single characoter, \"./folder/*\" exclude everything in the folder, recursively. Multiple patterns can be specified. Don't forget to add \"\" around the pattern.",
             action="append",
             dest="exclude",
             default=[])
-    parser.add_option("-X", "--xml",
+    parser.add_argument("-X", "--xml",
             help="Generate XML in cppncss style instead of the normal tabular output. Useful to generate report in Jenkins server",
             action="store_true",
             dest="xml",
             default=None)
-    parser.add_option("-P", "--no_preprocessor_count",
+    parser.add_argument("-P", "--no_preprocessor_count",
             help="By default, a #if will also increase the complexity. Adding this option to ignore them",
             action="store_true",
             dest="no_preprocessor_count",
             default=False)
-    parser.add_option("-t", "--working_threads",
+    parser.add_argument("-t", "--working_threads",
             help="number of working threads. The default value is 1.",
             action="store",
-            type="int",
+            type=int,
             dest="working_threads",
             default=1)
-    parser.add_option("-d", "--find_duplicates",
+    parser.add_argument("-d", "--find_duplicates",
             help="find and skip analysis for identical files. Will be made default in the next release",
             action="store_true",
             dest="duplicates",
             default=False)
-    parser.add_option("-e", "--display_fn_end_line",
+    parser.add_argument("-e", "--display_fn_end_line",
             help="display function end line number in addition to start line number. Will be made default in the next release",
             action="store_true",
             dest="display_fn_end_line",
             default=False)
-    parser.add_option("-m", "--modified",
+    parser.add_argument("-m", "--modified",
             help="Calculate modified cyclomatic complexity number",
             action="store_true",
             dest="switchCasesAsOneCondition",
             default=False)
-    parser.add_option("-E", "--extension",
+    parser.add_argument("-E", "--extension",
             help="under construction...", #"Use extension. Can be WordCount.",
             action="append",
             dest="extensions",
             default=[])
+    parser.add_argument("-s", "--sort",
+            help="Sort the warning with field. The field can be nloc, cyclomatic_complexity, token_count, parameter_count, etc. Or an customized file.",
+            action="append",
+            dest="sorting",
+            default=[])
 
     parser.usage = "lizard [options] [PATH or FILE] [PATH] ... "
     parser.description = __doc__
@@ -146,9 +122,6 @@ def createCommandLineParser():
 
 
 class FunctionInfo(object):
-    '''
-    Statistic information of a function.
-    '''
 
     def __init__(self, name, start_line = 0, ccn = 1):
         self.cyclomatic_complexity = ccn
@@ -177,10 +150,6 @@ class FunctionInfo(object):
 
 
 class FileInformation(object):
-    ''' 
-    Statistic information of a source file.
-    Including all the functions and the file summary.
-    '''
 
     def __init__(self, filename, nloc, function_list):
         self.filename = filename
@@ -217,17 +186,6 @@ class CodeInfoContext(object):
         self.START_NEW_FUNCTION('')
         self.forgive = False
 
-    def _new_line(self):
-        self.fileinfo.nloc += 1
-        self.newline = True
-
-    def _count_token(self):
-        self.fileinfo.token_count+=1
-        if self.newline:
-            self.current_function.nloc += 1
-            self.newline = False
-        self.current_function.token_count += 1
-
     def START_NEW_FUNCTION(self, name):
         self.newline = True
         self.current_function = FunctionInfo(name, self.current_line)
@@ -252,48 +210,93 @@ class CodeInfoContext(object):
         self.START_NEW_FUNCTION('')
 
 
-class LineCounter():
+class Preprocessor(object):
 
-    def extend_tokens(self, tokens, context):
-        self.context = context
-        self.context.current_line = 1 
+    macro_pattern = re.compile(r"#\s*(\w+)\s*(.*)", re.M | re.S)
+
+    @staticmethod
+    def extend_tokens(tokens, context):
+        # handle c preprocessors
         for token in tokens:
-            self.context.current_line += 1 if token == '\n' else (len(token.splitlines()) - 1)
-            if token == "\n":
-                self.context._new_line()
-            elif token.startswith("/*") or token.startswith("//"):
-                if len(token.splitlines()) > 1:
-                    self.context._new_line()
-                if token[2:].strip().startswith("#lizard forgive"):
-                    self.context.forgive = True
+            m = Preprocessor.macro_pattern.match(token)
+            if m:
+                yield "#" + m.group(1)
+                param = m.group(2).strip()
+                if param:
+                    yield param
             else:
-                self.context._count_token()
                 yield token
 
 
-class LanguageReaderBase(object):
+class LineCounter(object):
+    ''' And extension of lizard, to count the line and nloc.
+    '''
 
-    def process_tokens(self, tokens, context):
-        self._state = self._GLOBAL
-        self.context = context
+    FUNCTION_CAPTION = "  NLOC  "
+    FUNCTION_INFO_PART = "nloc"
+
+
+    def extend_tokens(self, tokens, context):
+        context.current_line = 1 
         for token in tokens:
-            self._state(token)
+            context.current_line += 1 if token == '\n' else (len(token.splitlines()) - 1)
+            if token == "\n":
+                self._new_line(context)
+            elif token.startswith("/*") or token.startswith("//"):
+                if len(token.splitlines()) > 1:
+                    self._new_line(context)
+                if token[2:].strip().startswith("#lizard forgive"):
+                    context.forgive = True
+            else:
+                self._count_token(context)
+                yield token
+
+    def _new_line(self, context):
+        context.fileinfo.nloc += 1
+        context.newline = True
+
+    def _count_token(self, context):
+        context.fileinfo.token_count+=1
+        if context.newline:
+            context.current_function.nloc += 1
+            context.newline = False
+        context.current_function.token_count += 1
+
+
+class TokenCounter(object):
+    ''' An extension that counts the token.
+        But because of historical and implementation reason, the token
+        counting is already done in LineCounter. So this class only do
+        the output.
+    '''
+
+    FUNCTION_CAPTION = " token "
+    FUNCTION_INFO_PART = "token_count"
 
     def extend_tokens(self, tokens, context):
         return tokens
 
 
-class CLikeReader(LanguageReaderBase):
-    '''
-    This is the reader for C, C++ and Java.
+class ParameterCounter(object):
+    ''' An extension that counts the parameters.
+        But because of historical and implementation reason, the parameter
+        counting is already done in FunctionCounter. So this class only do
+        the output.
     '''
 
-    def __init__(self, includeHashIfConditions = True, switchCasesAsOneCondition = False, **kwargs):
-        '''includeHashIfConditions: Should #if and #elseif have effect on the CCN
-           switchCasesAsOneCondition: Use modified CCN (i.e. for one switch case increases CCN by 1 instead of
-                        by the amount of case blocks)
-        '''
-        super(CLikeReader, self).__init__(**kwargs)
+    FUNCTION_CAPTION = " PARAM "
+    FUNCTION_INFO_PART = "parameter_count"
+
+    def extend_tokens(self, tokens, context):
+        return tokens
+
+
+class ConditionCounter(object):
+
+    FUNCTION_CAPTION = "  CCN  "
+    FUNCTION_INFO_PART = "cyclomatic_complexity"
+
+    def __init__(self, includeHashIfConditions = True, switchCasesAsOneCondition = False):
         self.conditions = set(
             ['if', 'for', 'while', '&&', '||', '?', 'catch'])
 
@@ -307,26 +310,50 @@ class CLikeReader(LanguageReaderBase):
         else:
             self.conditions.add('case')
 
+    def is_condition(self, token):
+        return token in self.conditions
+
+    def extend_tokens(self, tokens, context):
+        for token in tokens:
+            if self.is_condition(token):
+                context.CONDITION()
+            yield token
+
+
+#----------------------------------------------
+# CodeReaders are used to parse functions structures from code of different
+# language. Each language will need a subclass of CodeReader.
+
+class CodeReader(object):
+
+    @staticmethod
+    def compile_file_extension_re(*exts):
+        return re.compile(r".*\.(" + r"|".join(exts) + r")$", re.IGNORECASE)
+
+    @staticmethod
+    def get_reader(filename):
+        for lan in list(CodeReader.__subclasses__()):
+            if CodeReader.compile_file_extension_re(*lan.ext).match(filename):
+                return lan
+
+
+try: # lizard.py can run as a stand alone python script, without the extensions
+    from lizard_ext import JavaScriptReader
+except:
+    pass
+
+class CLikeReader(CodeReader):
+    '''
+    This is the reader for C, C++ and Java.
+    '''
+
+    ext = ["c", "cpp", "cc", "mm", "cxx", "h", "hpp"]
+
+    def __init__(self):
         self.bracket_level = 0
         self.br_count = 0
         self.last_preprocessor = None
-
-    macro_pattern = re.compile(r"#\s*(\w+)\s*(.*)", re.M | re.S)
-
-    def extend_tokens(self, tokens, context):
-        # handle c preprocessors
-        for token in tokens:
-            m = self.macro_pattern.match(token)
-            if m:
-                yield "#" + m.group(1)
-                param = m.group(2).strip()
-                if param:
-                    yield param
-            else:
-                yield token
-
-    def _is_condition(self, token):
-        return token in self.conditions
+        self._state = self._GLOBAL
 
     def _GLOBAL(self, token):
         if token == '(':
@@ -380,7 +407,7 @@ class CLikeReader(LanguageReaderBase):
     def _IMP(self, token):
         if token in ("#else", "#endif"):
             self.last_preprocessor = token
-        # will ignore the braces in a #else branch            
+        # will ignore the braces in a #else branch
         if self.last_preprocessor != '#else':
             if token == '{':
                 self.br_count += 1
@@ -389,14 +416,17 @@ class CLikeReader(LanguageReaderBase):
                 if self.br_count == 0:
                     self._state = self._GLOBAL
                     self.context.END_OF_FUNCTION()
-                    return
-        if self._is_condition(token):
-            self.context.CONDITION()
 
+class JavaReader(CLikeReader, CodeReader):
 
-class ObjCReader(CLikeReader):
-    def __init__(self, **kwargs):
-        super(ObjCReader, self).__init__(**kwargs)
+    ext = ['java']
+
+class ObjCReader(CLikeReader, CodeReader):
+
+    ext = ['m']
+
+    def __init__(self):
+        super(ObjCReader, self).__init__()
 
     def _DEC_TO_IMP(self, token):
         if token in ("+", "-"):
@@ -439,66 +469,40 @@ class ObjCReader(CLikeReader):
         self._state = self._OBJC_DEC
 
 
-def compile_file_extension_re(*exts):
-    return re.compile(r".*\.(" + r"|".join(exts) + r")$", re.IGNORECASE)
+class FunctionParser(object):
+    ''' FunctionParser parse source code into functions. This is different from language
+        to language. So FunctionParser need a language specific 'reader' to actually do
+        the job.
+        TODO: FunctionParser might yield functions in the future.
+    '''
 
-class LanguageChooser(object):
+    FUNCTION_CAPTION = " function@line@filename          "
 
-    def __init__(self, **readerArgs):
-        object.__init__(self)
-        self.readerArgs = readerArgs
+    def extend_tokens(self, tokens, context):
+        function_reader = (CodeReader.get_reader(context.fileinfo.filename) or CLikeReader)()
+        function_reader.context = context
+        for token in tokens:
+            function_reader._state(token)
 
-    lizard_language_infos = {
-                     'c/c++': {
-                          'name_pattern': compile_file_extension_re("c", "cpp", "cc", "mm", "cxx", "h", "hpp"),
-                          'reader':CLikeReader},
-                     'Java': {
-                          'name_pattern': compile_file_extension_re("java"),
-                          'reader':CLikeReader},
-                      'objC' : {
-                          'name_pattern': compile_file_extension_re("m"),
-                          'reader':ObjCReader}
-                    }
 
-    def get_language_by_filename(self, filename):
-        for lan in self.lizard_language_infos:
-            info = self.lizard_language_infos[lan]
-            if info['name_pattern'].match(filename):
-                return lan
+class FileAnalyzer(object):
 
-    def get_reader_by_file_name_otherwise_default(self, filename):
-        lan = self.get_language_by_filename(filename)
-        return self.lizard_language_infos[lan or "c/c++"]['reader'](**self.readerArgs)
-
-default_language_chooser = LanguageChooser(
-    includeHashIfConditions = True,
-    switchCasesAsOneCondition = False,
-)
-
-class FileAnalyzer:
-
-    def __init__(self, languageChooser = default_language_chooser, extensions = []):
-        self.extensions = extensions
-        self.languageChooser = languageChooser
+    def __init__(self, extensions = []):
+        self.processors = extensions
 
     def __call__(self, filename):
-        with open(filename) as f:
-            return self.analyze_source_code(filename, f.read())
+        try:
+            return self.analyze_source_code(filename, open(filename).read())
+        except IOError:
+            sys.stderr.write("Error: Fail to read source file '%s'"%filename)
 
     def analyze_source_code(self, filename, code):
-        reader = self.languageChooser.get_reader_by_file_name_otherwise_default(filename)
-        return self.analyze_source_code_with_parser(filename, code, reader)
-
-    def analyze_source_code_with_parser(self, filename, code, parser):
         tokens = generate_tokens(code)
         context = CodeInfoContext(filename)
-        for extension in [parser, LineCounter()] + self.extensions:
-            tokens = extension.extend_tokens(tokens, context)
-        parser.process_tokens(tokens, context)
+        for processor in self.processors:
+            tokens = processor.extend_tokens(tokens, context)
         return context.fileinfo
 
-analyze_file = FileAnalyzer()
-
 
 def warning_filter(option, fileStatistics):
     for file_info in fileStatistics:
@@ -559,7 +563,7 @@ class FreeFormattingTokenizer(object):
                            r"|&=|&&|&"+
                            r"|[!%^&\*\-=+\|\\<>/\]\+]+"+
                            r"|\n"+
-                           r"|\s+"+
+                           r"|[^\S\n]+"+
                            r"|.)", re.M | re.S)
 
     def __call__(self, source_code):
@@ -570,39 +574,46 @@ generate_tokens = FreeFormattingTokenizer()
 
 import sys
 
-def print_function_info_header():
-    print("==============================================================")
-    print("  nloc    CCN  token  param    function@line@file")
-    print("--------------------------------------------------------------")
+def print_function_info_header(extensions):
+    captions = "".join(ext.FUNCTION_CAPTION for ext in extensions if hasattr(ext, "FUNCTION_CAPTION"))
+    print("=" * len(captions))
+    print(captions)
+    print("-" * len(captions))
 
-def print_function_info(fun, filename, option):
+def print_function_info(fun, filename, extensions, option):
+    ll = ''
+    for ext in extensions:
+        if hasattr(ext, "FUNCTION_INFO_PART"):
+            ll += str(getattr(fun, ext.FUNCTION_INFO_PART)).rjust(len(ext.FUNCTION_CAPTION))
     line = '{}-{}'.format(fun.start_line, fun.end_line) if option.display_fn_end_line else fun.start_line
     output_params = {
         'nloc': fun.nloc,
         'CCN': fun.cyclomatic_complexity,
         'token': fun.token_count,
         'param': fun.parameter_count,
-        'name': fun.name,
+        'name': fun.long_name if option.verbose else fun.name,
         'line': line,
         'file': filename
     }
-    output_format = "%(nloc)6d %(CCN)6d %(token)6d %(param)6d    %(name)s@%(line)s@%(file)s"
-    if option.verbose:
-        output_params['name'] = fun.long_name
+
+    output_format = ll + " %(name)s@%(line)s@%(file)s"
     if option.warnings_only:
         output_format = "%(file)s:%(line)s: warning: %(name)s has %(CCN)d CCN and %(param)d params (%(nloc)d NLOC, %(token)d tokens)"
     print(output_format % output_params)
 
 def print_warnings(option, warnings):
     warning_count = 0
+    if type(option.sorting) is type([]) and len(option.sorting) > 0:
+        warnings = list(warnings)
+        warnings.sort(reverse = True, key=lambda x: getattr(x[0], option.sorting[0]))
     if not option.warnings_only:
         print(("\n" +
                "======================================\n" +
               "!!!! Warnings (CCN > %d) !!!!") % option.CCN)
-        print_function_info_header()
+        print_function_info_header(option.extensions)
     for warning in warnings:
         warning_count += 1
-        print_function_info(warning[0], warning[1], option)
+        print_function_info(warning[0], warning[1], option.extensions,  option)
 
     if warning_count == 0:
         print("No warning found. Excellent!")
@@ -641,14 +652,15 @@ def print_and_save_detail_information(allStatistics, option):
     if (option.warnings_only):
         all_functions = allStatistics
     else:
-        print_function_info_header()
+        print_function_info_header(option.extensions)
         for fileStatistics in allStatistics:
             for extension in option.extensions:
-                extension.reduce(fileStatistics)
+                if hasattr(extension, 'reduce'):
+                    extension.reduce(fileStatistics)
             if fileStatistics:
                 all_functions.append(fileStatistics)
                 for fun in fileStatistics.function_list:
-                    print_function_info(fun, fileStatistics.filename, option)
+                    print_function_info(fun, fileStatistics.filename, option.extensions, option)
 
         print("--------------------------------------------------------------")
         print("%d file analyzed." % (len(all_functions)))
@@ -673,7 +685,8 @@ def print_result(r, option):
     warning_count = print_warnings(option, warnings)
     print_total(warning_count, all_functions, option)
     for extension in option.extensions:
-        extension.print_result()
+        if hasattr(extension, 'print_result'):
+            extension.print_result()
     if option.number < warning_count:
         sys.exit(1)
 
@@ -836,7 +849,7 @@ import fnmatch
 import hashlib
 
 class FilesFilter(object):
-    
+
     def __init__(self, exclude_patterns, check_duplicates):
         self.exclude_patterns = exclude_patterns
         self.check_duplicates = check_duplicates
@@ -870,7 +883,7 @@ class FilesFilter(object):
         return code_md5.hexdigest()
 
     def _notExluded(self, str_to_match):
-        return LanguageChooser().get_language_by_filename(str_to_match) and \
+        return CodeReader.get_reader(str_to_match) and \
             all(not fnmatch.fnmatch(str_to_match, p) for p in self.exclude_patterns)
 
     def _notDuplicate(self, full_path_name):
@@ -884,25 +897,44 @@ class FilesFilter(object):
 
 def parse_args(argv):
 
-    def get_extensions(extension_names):
-        return [__import__('lizard' + name).LizardExtension() for name in extension_names]
-
     def get_whitelist():
         whitelist_filename = "whitelizard.txt"
         if os.path.isfile(whitelist_filename):
             return open(whitelist_filename, mode='r').read()
         return ''
 
-    options, args = createCommandLineParser().parse_args(args=argv)
+    options = createCommandLineParser(argv[0]).parse_args(args=argv[1:])
     options.whitelist = get_whitelist()
-    options.extensions = get_extensions(options.extensions)
-    paths = ["."] if len(args) == 1 else args[1:]
-    return paths, options
+    options.extensions = get_extensions(options.extensions, not options.no_preprocessor_count, options.switchCasesAsOneCondition)
+    function_parts = [getattr(ext, 'FUNCTION_INFO_PART') for ext in options.extensions
+            if hasattr(ext, 'FUNCTION_INFO_PART')]
+    for sort_factor in options.sorting:
+        if sort_factor not in function_parts:
+            error_message =  "Wrong sorting field '%s'.\n" % sort_factor
+            error_message +=  "Candidates are: " + str(function_parts) + "\n"
+            sys.stderr.write(error_message)
+            sys.exit(2)
+    return options
+
+def get_extensions(extension_names, countPreprocessor = True, switchCaseAsOneCondition = False):
+    from importlib import import_module
+    basicExtensions = [
+        Preprocessor,
+        LineCounter(),
+        ConditionCounter(countPreprocessor, switchCaseAsOneCondition),
+        TokenCounter(),
+        ParameterCounter()
+        ]
+    return basicExtensions +\
+        [import_module('lizard_ext.lizard' + name).LizardExtension() if type(name) == type("") else name  for name in extension_names] +\
+        [FunctionParser()]
+
+analyze_file = FileAnalyzer(get_extensions([]))
 
 def lizard_main(argv =  sys.argv):
-    paths, options = parse_args(argv)
+    options = parse_args(argv)
     printer = print_xml if options.xml else print_result
-    r = analyze(paths, options)
+    r = analyze(options.paths, options)
     printer(r, options)
 
 if __name__ == "__main__":
