@@ -2,7 +2,7 @@ import unittest
 from test.mock import patch
 import lizard
 from lizard import lizard_main
-import os
+import os, sys
 
 @patch('lizard.open', create=True)
 @patch.object(os, 'walk')
@@ -31,7 +31,7 @@ class TestApplication(unittest.TestCase):
             self.assertEqual(1, len(fileInfos))
             self.assertEqual('foo', fileInfos[0].function_list[0].name)
         os_walk.return_value = [('.', [], ['a.cpp'])]
-        mock_open.return_value.__enter__.return_value.read.return_value = "void foo(){}"
+        mock_open.return_value.read.return_value = "void foo(){}"
         print_result.side_effect = check_result
         lizard_main(['lizard'])
 
@@ -41,7 +41,7 @@ class TestApplication(unittest.TestCase):
             self.assertEqual(1, len(fileInfos))
             self.assertEqual('foo', fileInfos[0].function_list[0].name)
         os_walk.return_value = [('.', [], ['a.cpp'])]
-        mock_open.return_value.__enter__.return_value.read.return_value = "void foo(){}"
+        mock_open.return_value.read.return_value = "void foo(){}"
         print_result.side_effect = check_result
         lizard_main(['lizard'])
 
@@ -73,7 +73,7 @@ class TestOptions(unittest.TestCase):
         def store_result(result, options):
             self.fileInfos = list(result) 
         os_walk.return_value = [('.', [], ['a.cpp'])]
-        mock_open.return_value.__enter__.return_value.read.return_value = self.source_code
+        mock_open.return_value.read.return_value = self.source_code
         print_result.side_effect = store_result
         lizard_main(argv)
         
@@ -102,7 +102,7 @@ class TestOptionParsing(unittest.TestCase):
     def test_load_whitelist_from_file(self, mock_open, isfile):
         isfile.return_value = True
         mock_open.return_value.read.return_value = "foo"
-        _, options = parse_args(['lizard'])
+        options = parse_args(['lizard'])
         self.assertEqual("foo", options.whitelist)
         isfile.assert_called_with('whitelizard.txt')
         mock_open.assert_called_with('whitelizard.txt', mode='r')
@@ -111,6 +111,24 @@ class TestOptionParsing(unittest.TestCase):
     @patch('lizard.open', create=True)
     def test_should_be_empty_if_whitelist_file_doesnot_exist(self, mock_open, isfile):
         isfile.return_value = False
-        _, options = parse_args(['lizard'])
+        options = parse_args(['lizard'])
         self.assertEqual("", options.whitelist)
 
+    def test_default_sorting(self):
+        options = parse_args(['lizard'])
+        self.assertEqual(0, len(options.sorting))
+
+    def test_sorting_factor(self):
+        options = parse_args(['lizard', '-snloc'])
+        self.assertEqual("nloc", options.sorting[0])
+
+    @patch.object(sys, 'exit')
+    def test_sorting_factor_does_not_exist(self, mock_exit):
+        options = parse_args(['lizard', '-sdoesnotexist'])
+        mock_exit.assert_called_with(2)
+
+    @patch.object(sys, 'exit')
+    @patch('sys.stderr')
+    def test_sorting_factor_does_not_exist_error_message(self, mock_stderr, mock_exit):
+        options = parse_args(['lizard', '-sdoesnotexist'])
+        mock_stderr.write.assert_called_with("Wrong sorting field 'doesnotexist'.\nCandidates are: ['nloc', 'cyclomatic_complexity', 'token_count', 'parameter_count']\n")
