@@ -2,7 +2,7 @@ import unittest
 from test.mock import Mock, patch
 import sys
 from lizard import print_warnings, print_and_save_detail_information, FunctionInfo, FileInformation,\
-    print_result, XMLFormatter
+    print_result, XMLFormatter, get_extensions
 
 class StreamStdoutTestCase(unittest.TestCase):
     def setUp(self):
@@ -23,15 +23,43 @@ class StreamStdoutTestCase(unittest.TestCase):
         def __getattr__(self, attr):
             return getattr(self.stream, attr)    
 
+class TestFunctionOutput(StreamStdoutTestCase):
+
+    def setUp(self):
+        StreamStdoutTestCase.setUp(self)
+        self.options = Mock(warnings_only=False, extensions = get_extensions([]))
+
+    def test_function_info_header_should_have_a_box(self):
+        print_and_save_detail_information([], self.options)
+        self.assertIn("=" * 20, sys.stdout.stream.splitlines()[0])
+
+    def test_function_info_header_should_have_the_captions(self):
+        print_and_save_detail_information([], self.options)
+        self.assertEquals("  NLOC    CCN   token  PARAM  function@line@filename          ", sys.stdout.stream.splitlines()[1])
+
+    def test_function_info_header_should_have_the_captions_of_external_extensions(self):
+        external_extension = Mock(FUNCTION_CAPTION = "*external_extension*")
+        self.options = Mock(warnings_only=False, extensions = get_extensions([external_extension]))
+        print_and_save_detail_information([], self.options)
+        self.assertEquals("  NLOC    CCN   token  PARAM *external_extension* function@line@filename          ", sys.stdout.stream.splitlines()[1])
+
+    def test_print_fileinfo(self):
+        fun = FunctionInfo("foo", 100)
+        fun.end_line = 100
+        fun.cyclomatic_complexity = 16
+        fileStat = FileInformation("FILENAME", 1, [fun])
+        print_and_save_detail_information([fileStat], self.options)
+        self.assertEquals("       0     16      1      0 foo@100-100@FILENAME", sys.stdout.stream.splitlines()[3])
+
 class TestWarningOutput(StreamStdoutTestCase):
         
     def test_should_have_header_when_warning_only_is_off(self):
-        option = Mock(warnings_only=False, CCN=15)
+        option = Mock(warnings_only=False, CCN=15, extensions = [])
         print_warnings(option, [])
         self.assertIn("Warnings (CCN > 15)", sys.stdout.stream)
 
     def test_should_say_no_warning_when_warning_only_is_off(self):
-        option = Mock(warnings_only=False, CCN=15)
+        option = Mock(warnings_only=False, CCN=15, extensions = [])
         print_warnings(option, [])
         self.assertIn("No warning found. Excellent!\n", sys.stdout.stream)
 
@@ -41,7 +69,7 @@ class TestWarningOutput(StreamStdoutTestCase):
         self.assertNotIn("Warnings (CCN > 15)", sys.stdout.stream)
 
     def test_should_use_clang_format_for_warning(self):
-        option = Mock(display_fn_end_line = False)
+        option = Mock(display_fn_end_line = False, extensions = get_extensions([]))
         print_warnings(option, [(FunctionInfo("foo", 100), "FILENAME")])
         self.assertIn("FILENAME:100: warning: foo has 1 CCN and 0 params (0 NLOC, 1 tokens)\n", sys.stdout.stream)
 
@@ -50,10 +78,26 @@ class TestWarningOutput(StreamStdoutTestCase):
         fun.end_line = 100
         fun.cyclomatic_complexity = 16
         fileStat = FileInformation("FILENAME", 1, [fun])
-        option = Mock(display_fn_end_line = True)
+        option = Mock(display_fn_end_line = True, extensions = get_extensions([])) 
         print_warnings(option, [(fun, "FILENAME")])
         self.assertIn("FILENAME:100-100: warning: foo has 16 CCN and 0 params (0 NLOC, 1 tokens)\n", sys.stdout.stream)
 
+    @patch('lizard.print_function_info')
+    def test_sort_warning(self, print_function_info):
+        option = Mock(display_fn_end_line = False, extensions = get_extensions([]))
+        option.sorting = ['cyclomatic_complexity']
+        foo = FunctionInfo("foo", 100)
+        foo.cyclomatic_complexity = 10
+        bar = FunctionInfo("bar", 100)
+        bar.cyclomatic_complexity = 15
+        print_warnings(option, [(foo, "FILENAME"),(bar, "FILENAME")])
+        self.assertEqual('bar', print_function_info.call_args_list[0][0][0].name)
+
+    def test_sort_warning_with_generator(self):
+        option = Mock(display_fn_end_line = False, extensions = get_extensions([]))
+        option.sorting = ['cyclomatic_complexity']
+        print_warnings(option, (x for x in []))
+
 
 class TestFileOutput(StreamStdoutTestCase):
     
@@ -70,14 +114,19 @@ class TestFileOutput(StreamStdoutTestCase):
 
     
 class TestAllOutput(StreamStdoutTestCase):
-        
     def test_print_extension_results(self):
+
         file_infos = []
         extension = Mock()
         option = Mock(CCN=15, number = 0, extensions = [extension], whitelist='')
         print_result(file_infos, option)
         self.assertEqual(1, extension.print_result.call_count)
 
+    def test_should_not_print_extension_results_when_not_implemented(self):
+        file_infos = []
+        option = Mock(CCN=15, number = 0, extensions = [object()], whitelist='')
+        print_result(file_infos, option)
+
     @patch.object(sys, 'exit')
     def test_print_result(self, mock_exit):
         file_infos = [FileInformation('f1.c', 1, []), FileInformation('f2.c', 1, [])]
