@@ -4,7 +4,7 @@
 import unittest
 import sys
 from test.mock import patch, Mock
-from lizard import FileAnalyzer, CLikeReader, mapFilesToAnalyzer, FunctionInfo, analyze_file
+from lizard import analyze_file, CLikeReader, mapFilesToAnalyzer, FunctionInfo, analyze_file
 
 
 def analyzer_mock(filename):
@@ -50,23 +50,20 @@ class Test_analyze_files(unittest.TestCase):
 class Test_FileAnalyzer(unittest.TestCase):
     
     def setUp(self):
-        self.analyzer = FileAnalyzer()
+        self.analyzer = analyze_file
         
     def test_analyze_c_file(self, mock_open):
-        file_handle = mock_open.return_value.__enter__.return_value
-        file_handle.read.return_value = "int foo(){haha();\n}"
+        file_handle = mock_open.return_value.read.return_value = "int foo(){haha();\n}"
         r = mapFilesToAnalyzer(["f1.c"], self.analyzer, 1)
         self.assertEqual(1, len([x for x in r]))
         
     def test_analyze_c_file_with_multiple_thread(self, mock_open):
-        file_handle = mock_open.return_value.__enter__.return_value
-        file_handle.read.return_value = "int foo(){haha();\n}"
+        file_handle = mock_open.return_value.read.return_value = "int foo(){haha();\n}"
         r = mapFilesToAnalyzer(["f1.c"], self.analyzer, 2)
         self.assertEqual(1, len([x for x in r]))
     
     def test_fileInfomation(self, mock_open):
-        file_handle = mock_open.return_value.__enter__.return_value
-        file_handle.read.return_value = "int foo(){haha();\n}"
+        mock_open.return_value.read.return_value = "int foo(){haha();\n}"
         r = mapFilesToAnalyzer(["f1.c"], self.analyzer, 1)
         fileInfo = list(r)[0]
         self.assertEqual(1, fileInfo.nloc)
@@ -74,6 +71,13 @@ class Test_FileAnalyzer(unittest.TestCase):
         self.assertEqual(1, fileInfo.average_CCN)
         self.assertEqual(9, fileInfo.average_token)
 
+    @patch.object(sys, 'stderr')
+    def test_should_report_when_having_other_problem_and_continue(self, mock_stderr, mock_open):
+        mock_open.side_effect = IOError("[Errno 2] No such file or directory")
+        analyze_file("f1.c")
+        self.assertEqual(1, mock_stderr.write.call_count)
+        error_message = mock_stderr.write.call_args[0][0]
+        self.assertIn("Error", error_message)
 
 class Test_FunctionInfo(unittest.TestCase):
 
@@ -163,23 +167,6 @@ class Test_Exclude_Patterns(unittest.TestCase):
         self.assertEqual(["./f1.cpp", "./f2.cpp"], list(files))
 
 
-from lizard import LanguageChooser
-class TestLanguageChooser(unittest.TestCase):
-    
-    def test_not_case_sensitive(self):
-        self.assertEqual("c/c++", LanguageChooser().get_language_by_filename("a.Cpp"));
-    
-    def test_java(self):
-        self.assertEqual("Java", LanguageChooser().get_language_by_filename("a.java"));
-    
-    def test_objectiveC(self):
-        self.assertEqual("objC", LanguageChooser().get_language_by_filename("a.m"));
-    
-    def test_c_cpp(self):
-        for name in ("a.cpp", ".cxx", ".h", ".hpp"):
-            self.assertEqual("c/c++", LanguageChooser().get_language_by_filename(name),
-                             "File name '%s' is not recognized as c/c++ file" % name);
-    
 from lizard import warning_filter, FileInformation, Whitelist
 
 class TestWarningFilter(unittest.TestCase):
